@@ -1,23 +1,10 @@
 /*
-   This file is part of the KDE libraries
-   Copyright (c) 2006, 2007 Thomas Braxton <kde.braxton@gmail.com>
-   Copyright (c) 1999 Preston Brown <pbrown@kde.org>
-   Copyright (c) 1997 Matthias Kalle Dalheimer <kalle@kde.org>
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 2006, 2007 Thomas Braxton <kde.braxton@gmail.com>
+    SPDX-FileCopyrightText: 1999 Preston Brown <pbrown@kde.org>
+    SPDX-FileCopyrightText: 1997 Matthias Kalle Dalheimer <kalle@kde.org>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "kconfiggroup.h"
@@ -25,18 +12,19 @@
 
 #include "kconfig.h"
 #include "kconfig_p.h"
+#include "kconfig_core_log_settings.h"
 #include "ksharedconfig.h"
 #include "kconfigdata.h"
 
-#include <QtCore/QDate>
-#include <QtCore/QSharedData>
-#include <QtCore/QFile>
-#include <QtCore/QPoint>
-#include <QtCore/QRect>
-#include <QtCore/QString>
-#include <QtCore/QTextStream>
-#include <QtCore/QDir>
-#include <QtCore/QUrl>
+#include <QDate>
+#include <QSharedData>
+#include <QFile>
+#include <QPoint>
+#include <QRect>
+#include <QString>
+#include <QTextStream>
+#include <QDir>
+#include <QUrl>
 
 #include <stdlib.h>
 
@@ -182,19 +170,25 @@ QStringList KConfigGroupPrivate::deserializeList(const QString &data)
     return value;
 }
 
-static QList<int> asIntList(const QByteArray &string)
+static QVector<int> asIntList(const QByteArray &string)
 {
-    QList<int> list;
-    Q_FOREACH (const QByteArray &s, string.split(',')) {
+    const auto &splitString = string.split(',');
+
+    QVector<int> list;
+    list.reserve(splitString.count());
+    for (const QByteArray &s : splitString) {
         list << s.toInt();
     }
     return list;
 }
 
-static QList<qreal> asRealList(const QByteArray &string)
+static QVector<qreal> asRealList(const QByteArray &string)
 {
-    QList<qreal> list;
-    Q_FOREACH (const QByteArray &s, string.split(',')) {
+    const auto &splitString = string.split(',');
+
+    QVector<qreal> list;
+    list.reserve(splitString.count());
+    for (const QByteArray &s : splitString) {
         list << s.toDouble();
     }
     return list;
@@ -251,89 +245,89 @@ QVariant KConfigGroup::convertToQVariant(const char *pKey, const QByteArray &val
         return tmp;
     }
     case QMetaType::QPoint: {
-        const QList<int> list = asIntList(value);
+        const auto list = asIntList(value);
 
         if (list.count() != 2) {
-            qWarning() << errString(pKey, value, aDefault)
+            qCWarning(KCONFIG_CORE_LOG) << errString(pKey, value, aDefault)
                        << formatError(2, list.count());
             return aDefault;
         }
         return QPoint(list.at(0), list.at(1));
     }
     case QMetaType::QPointF: {
-        const QList<qreal> list = asRealList(value);
+        const auto list = asRealList(value);
 
         if (list.count() != 2) {
-            qWarning() << errString(pKey, value, aDefault)
+            qCWarning(KCONFIG_CORE_LOG) << errString(pKey, value, aDefault)
                        << formatError(2, list.count());
             return aDefault;
         }
         return QPointF(list.at(0), list.at(1));
     }
     case QMetaType::QRect: {
-        const QList<int> list = asIntList(value);
+        const auto list = asIntList(value);
 
         if (list.count() != 4) {
-            qWarning() << errString(pKey, value, aDefault)
+            qCWarning(KCONFIG_CORE_LOG) << errString(pKey, value, aDefault)
                        << formatError(4, list.count());
             return aDefault;
         }
         const QRect rect(list.at(0), list.at(1), list.at(2), list.at(3));
         if (!rect.isValid()) {
-            qWarning() << errString(pKey, value, aDefault);
+            qCWarning(KCONFIG_CORE_LOG) << errString(pKey, value, aDefault);
             return aDefault;
         }
         return rect;
     }
     case QMetaType::QRectF: {
-        const QList<qreal> list = asRealList(value);
+        const auto list = asRealList(value);
 
         if (list.count() != 4) {
-            qWarning() << errString(pKey, value, aDefault)
+            qCWarning(KCONFIG_CORE_LOG) << errString(pKey, value, aDefault)
                        << formatError(4, list.count());
             return aDefault;
         }
         const QRectF rect(list.at(0), list.at(1), list.at(2), list.at(3));
         if (!rect.isValid()) {
-            qWarning() << errString(pKey, value, aDefault);
+            qCWarning(KCONFIG_CORE_LOG) << errString(pKey, value, aDefault);
             return aDefault;
         }
         return rect;
     }
     case QMetaType::QSize: {
-        const QList<int> list = asIntList(value);
+        const auto list = asIntList(value);
 
         if (list.count() != 2) {
-            qWarning() << errString(pKey, value, aDefault)
+            qCWarning(KCONFIG_CORE_LOG) << errString(pKey, value, aDefault)
                        << formatError(2, list.count());
             return aDefault;
         }
         const QSize size(list.at(0), list.at(1));
         if (!size.isValid()) {
-            qWarning() << errString(pKey, value, aDefault);
+            qCWarning(KCONFIG_CORE_LOG) << errString(pKey, value, aDefault);
             return aDefault;
         }
         return size;
     }
     case QMetaType::QSizeF: {
-        const QList<qreal> list = asRealList(value);
+        const auto list = asRealList(value);
 
         if (list.count() != 2) {
-            qWarning() << errString(pKey, value, aDefault)
+            qCWarning(KCONFIG_CORE_LOG) << errString(pKey, value, aDefault)
                        << formatError(2, list.count());
             return aDefault;
         }
         const QSizeF size(list.at(0), list.at(1));
         if (!size.isValid()) {
-            qWarning() << errString(pKey, value, aDefault);
+            qCWarning(KCONFIG_CORE_LOG) << errString(pKey, value, aDefault);
             return aDefault;
         }
         return size;
     }
     case QMetaType::QDateTime: {
-        const QList<int> list = asIntList(value);
+        const auto list = asIntList(value);
         if (list.count() != 6) {
-            qWarning() << errString(pKey, value, aDefault)
+            qCWarning(KCONFIG_CORE_LOG) << errString(pKey, value, aDefault)
                        << formatError(6, list.count());
             return aDefault;
         }
@@ -341,31 +335,31 @@ QVariant KConfigGroup::convertToQVariant(const char *pKey, const QByteArray &val
         const QTime time(list.at(3), list.at(4), list.at(5));
         const QDateTime dt(date, time);
         if (!dt.isValid()) {
-            qWarning() << errString(pKey, value, aDefault);
+            qCWarning(KCONFIG_CORE_LOG) << errString(pKey, value, aDefault);
             return aDefault;
         }
         return dt;
     }
     case QMetaType::QDate: {
-        QList<int> list = asIntList(value);
+        auto list = asIntList(value);
         if (list.count() == 6) {
             list = list.mid(0, 3);    // don't break config files that stored QDate as QDateTime
         }
         if (list.count() != 3) {
-            qWarning() << errString(pKey, value, aDefault)
+            qCWarning(KCONFIG_CORE_LOG) << errString(pKey, value, aDefault)
                        << formatError(3, list.count());
             return aDefault;
         }
         const QDate date(list.at(0), list.at(1), list.at(2));
         if (!date.isValid()) {
-            qWarning() << errString(pKey, value, aDefault);
+            qCWarning(KCONFIG_CORE_LOG) << errString(pKey, value, aDefault);
             return aDefault;
         }
         return date;
     }
     case QMetaType::QColor:
     case QMetaType::QFont:
-        qWarning() << "KConfigGroup::readEntry was passed GUI type '"
+        qCWarning(KCONFIG_CORE_LOG) << "KConfigGroup::readEntry was passed GUI type '"
                    << aDefault.typeName()
                    << "' but KConfigGui isn't linked! If it is linked to your program, "
                    "this is a platform bug. Please inform the KDE developers";
@@ -377,12 +371,12 @@ QVariant KConfigGroup::convertToQVariant(const char *pKey, const QByteArray &val
         break;
     }
 
-    qWarning() << "unhandled type " << aDefault.typeName();
+    qCWarning(KCONFIG_CORE_LOG) << "unhandled type " << aDefault.typeName();
     return QVariant();
 }
 
 #ifdef Q_OS_WIN
-# include <QtCore/QDir>
+# include <QDir>
 #endif
 
 static bool cleanHomeDirPath(QString &path, const QString &homeDir)
@@ -567,7 +561,7 @@ void KConfigGroup::deleteGroup(WriteConfigFlags flags)
     config()->deleteGroup(d->fullName(), flags);
 }
 
-#ifndef KDE_NO_DEPRECATED
+#if KCONFIGCORE_BUILD_DEPRECATED_SINCE(5, 0)
 void KConfigGroup::changeGroup(const QString &group)
 {
     Q_ASSERT_X(isValid(), "KConfigGroup::changeGroup", "accessing an invalid group");
@@ -576,7 +570,7 @@ void KConfigGroup::changeGroup(const QString &group)
 }
 #endif
 
-#ifndef KDE_NO_DEPRECATED
+#if KCONFIGCORE_BUILD_DEPRECATED_SINCE(5, 0)
 void KConfigGroup::changeGroup(const char *group)
 {
     Q_ASSERT_X(isValid(), "KConfigGroup::changeGroup", "accessing an invalid group");
@@ -743,8 +737,11 @@ QVariantList KConfigGroup::readEntry(const char *key, const QVariantList &aDefau
         return aDefault;
     }
 
+    const auto &list = KConfigGroupPrivate::deserializeList(data);
+
     QVariantList value;
-    Q_FOREACH (const QString &v, KConfigGroupPrivate::deserializeList(data)) {
+    value.reserve(list.count());
+    for (const QString &v : list) {
         value << v;
     }
 
@@ -880,8 +877,9 @@ void KConfigGroup::writeEntry(const char *key, const QStringList &list, WriteCon
     Q_ASSERT_X(!d->bConst, "KConfigGroup::writeEntry", "writing to a read-only group");
 
     QList<QByteArray> balist;
+    balist.reserve(list.count());
 
-    Q_FOREACH (const QString &entry, list) {
+    for (const QString &entry : list) {
         balist.append(entry.toUtf8());
     }
 
@@ -899,8 +897,9 @@ void KConfigGroup::writeEntry(const char *key, const QVariantList &list, WriteCo
     Q_ASSERT_X(!d->bConst, "KConfigGroup::writeEntry", "writing to a read-only group");
 
     QList<QByteArray> data;
+    data.reserve(list.count());
 
-    Q_FOREACH (const QVariant &v, list) {
+    for (const QVariant &v : list) {
         if (v.type() == QVariant::ByteArray) {
             data << v.toByteArray();
         } else {
@@ -944,94 +943,109 @@ void KConfigGroup::writeEntry(const char *key, const QVariant &value,
         break;
     case QMetaType::QVariantList:
         if (!value.canConvert(QMetaType::QStringList))
-            qWarning() << "not all types in \"" << key << "\" can convert to QString,"
+            qCWarning(KCONFIG_CORE_LOG) << "not all types in \"" << key << "\" can convert to QString,"
                        " information will be lost";
+        Q_FALLTHROUGH();
     case QMetaType::QStringList:
         writeEntry(key, value.toList(), flags);
         return;
     case QMetaType::QPoint: {
-        QVariantList list;
         const QPoint rPoint = value.toPoint();
-        list.insert(0, rPoint.x());
-        list.insert(1, rPoint.y());
+
+        const QVariantList list{
+            rPoint.x(),
+            rPoint.y()
+        };
 
         writeEntry(key, list, flags);
         return;
     }
     case QMetaType::QPointF: {
-        QVariantList list;
         const QPointF point = value.toPointF();
-        list.insert(0, point.x());
-        list.insert(1, point.y());
+
+        const QVariantList list{
+            point.x(),
+            point.y()
+        };
 
         writeEntry(key, list, flags);
         return;
     }
     case QMetaType::QRect: {
-        QVariantList list;
         const QRect rRect = value.toRect();
-        list.insert(0, rRect.left());
-        list.insert(1, rRect.top());
-        list.insert(2, rRect.width());
-        list.insert(3, rRect.height());
+
+        const QVariantList list{
+            rRect.left(),
+            rRect.top(),
+            rRect.width(),
+            rRect.height()
+        };
 
         writeEntry(key, list, flags);
         return;
     }
     case QMetaType::QRectF: {
-        QVariantList list;
         const QRectF rRectF = value.toRectF();
-        list.insert(0, rRectF.left());
-        list.insert(1, rRectF.top());
-        list.insert(2, rRectF.width());
-        list.insert(3, rRectF.height());
+
+        const QVariantList list{
+            rRectF.left(),
+            rRectF.top(),
+            rRectF.width(),
+            rRectF.height()
+        };
 
         writeEntry(key, list, flags);
         return;
     }
     case QMetaType::QSize: {
-        QVariantList list;
         const QSize rSize = value.toSize();
-        list.insert(0, rSize.width());
-        list.insert(1, rSize.height());
+
+        const QVariantList list{
+            rSize.width(),
+            rSize.height()
+        };
 
         writeEntry(key, list, flags);
         return;
     }
     case QMetaType::QSizeF: {
-        QVariantList list;
         const QSizeF rSizeF = value.toSizeF();
-        list.insert(0, rSizeF.width());
-        list.insert(1, rSizeF.height());
+
+        const QVariantList list{
+            rSizeF.width(),
+            rSizeF.height()
+        };
 
         writeEntry(key, list, flags);
         return;
     }
     case QMetaType::QDate: {
-        QVariantList list;
         const QDate date = value.toDate();
 
-        list.insert(0, date.year());
-        list.insert(1, date.month());
-        list.insert(2, date.day());
+        const QVariantList list{
+            date.year(),
+            date.month(),
+            date.day()
+        };
 
         writeEntry(key, list, flags);
         return;
     }
     case QMetaType::QDateTime: {
-        QVariantList list;
         const QDateTime rDateTime = value.toDateTime();
 
         const QTime time = rDateTime.time();
         const QDate date = rDateTime.date();
 
-        list.insert(0, date.year());
-        list.insert(1, date.month());
-        list.insert(2, date.day());
+        const QVariantList list{
+            date.year(),
+            date.month(),
+            date.day(),
 
-        list.insert(3, time.hour());
-        list.insert(4, time.minute());
-        list.insert(5, time.second());
+            time.hour(),
+            time.minute(),
+            time.second(),
+        };
 
         writeEntry(key, list, flags);
         return;
@@ -1039,7 +1053,7 @@ void KConfigGroup::writeEntry(const char *key, const QVariant &value,
 
     case QMetaType::QColor:
     case QMetaType::QFont:
-        qWarning() << "KConfigGroup::writeEntry was passed GUI type '"
+        qCWarning(KCONFIG_CORE_LOG) << "KConfigGroup::writeEntry was passed GUI type '"
                    << value.typeName()
                    << "' but KConfigGui isn't linked! If it is linked to your program, this is a platform bug. "
                    "Please inform the KDE developers";
@@ -1048,7 +1062,7 @@ void KConfigGroup::writeEntry(const char *key, const QVariant &value,
         data = QUrl(value.toUrl()).toString().toUtf8();
         break;
     default:
-        qWarning() << "KConfigGroup::writeEntry - unhandled type" << value.typeName() << "in group" << name();
+        qCWarning(KCONFIG_CORE_LOG) << "KConfigGroup::writeEntry - unhandled type" << value.typeName() << "in group" << name();
     }
 
     writeEntry(key, data, flags);
@@ -1083,7 +1097,7 @@ void KConfigGroup::writeXdgListEntry(const char *key, const QStringList &list, W
     const QStringList::ConstIterator end = list.constEnd();
     for (; it != end; ++it) {
         QString val(*it);
-        val.replace(QLatin1Char('\\'), QStringLiteral("\\\\")).replace(QLatin1Char(';'), QStringLiteral("\\;"));
+        val.replace(QLatin1Char('\\'), QLatin1String("\\\\")).replace(QLatin1Char(';'), QLatin1String("\\;"));
         value += val;
         value += QLatin1Char(';');
     }
@@ -1115,7 +1129,8 @@ void KConfigGroup::writePathEntry(const char *pKey, const QStringList &value, Wr
     Q_ASSERT_X(!d->bConst, "KConfigGroup::writePathEntry", "writing to a read-only group");
 
     QList<QByteArray> list;
-    Q_FOREACH (const QString &path, value) {
+    list.reserve(value.length());
+    for (const QString &path : value) {
         list << translatePath(path).toUtf8();
     }
 
@@ -1136,16 +1151,26 @@ void KConfigGroup::deleteEntry(const QString &key, WriteConfigFlags flags)
 }
 
 void KConfigGroup::revertToDefault(const char *key)
+{
+    revertToDefault(key, WriteConfigFlags());
+}
+
+void KConfigGroup::revertToDefault(const char *key, WriteConfigFlags flags)
 {
     Q_ASSERT_X(isValid(), "KConfigGroup::revertToDefault", "accessing an invalid group");
     Q_ASSERT_X(!d->bConst, "KConfigGroup::revertToDefault", "writing to a read-only group");
 
-    config()->d_func()->revertEntry(d->fullName(), key);
+    config()->d_func()->revertEntry(d->fullName(), key, flags);
 }
 
 void KConfigGroup::revertToDefault(const QString &key)
 {
-    revertToDefault(key.toUtf8().constData());
+    revertToDefault(key, WriteConfigFlags());
+}
+
+void KConfigGroup::revertToDefault(const QString &key, WriteConfigFlags flags)
+{
+    revertToDefault(key.toUtf8().constData(), flags);
 }
 
 bool KConfigGroup::hasDefault(const char *key) const
